#include <hidef.h>      /* common defines and macros */
#include <MC9S12XEP100.h>     /* derivative information */
#pragma LINK_INFO DERIVATIVE "mc9s12xep100"
#include "mscan.h"

// Unsecured status flash
//const unsigned char flash_security  @0xFF0F = 0xFE;

#define     _TARGET     0     // 0 = GET
                              // 1 = SEND
                                

struct can_msg buf_send, buf_get;

unsigned int i;



#pragma CODE_SEG NON_BANKED
interrupt 52 void CAN_WAKE(void)
{
  while(1){
    PORTB = ~PORTB;
    for(i=0; i<50000; i++){
      asm nop;
      asm nop;
      asm nop;
    }
  }
}
#pragma CODE_SEG DEFAULT





/////////////////////////////////////////////////////////////////////////////////////////
// TPeriphInit
// --------------------------------------------------------------------------------------
// Peripheral Initialization for Test
/////////////////////////////////////////////////////////////////////////////////////////
void TPeriphInit(void)
{
  
  // Configures PA[3..0] port as output  
  PORTA = 0x0F;                                                    
  DDRA = 0x0F;                        

  DDRB = 0xFF;                                        

  // Configures PortB as input                                   
  //DDRB = 0x00;                                        
  // Enables pull-ups on portB
  //PUCR |= 0x02;                                   

  // Configures the ATD peripheral
  // 8 bit data resolution
  ATD0CTL1 = 0x10; 
  // Left justified data, 2 conversion sequence and non-FIFO mode
  ATD0CTL3 = 0x13;
  // fBUS=2MHz, fATDCLK = 1 MHz (PRESCLAER = 0) Select 24 Sample Time
  ATD0CTL4 = 0xE0;
  
  // Configures the PIT channel 0
  PITCE = 0x01;                               
  PITLD0 = 1000;
  
  MSCANOpenCommunication(CAN_4);
     
  EnableInterrupts;
  
}

/////////////////////////////////////////////////////////////////////////////////////////
// Delay
// --------------------------------------------------------------------------------------
// Delay of multiples of 500 us
/////////////////////////////////////////////////////////////////////////////////////////
void Delay(unsigned int del)
{
  //Start Timer on channel 0                                
  PITCFLMT = 0xA0;
  while(del)
  {
    if(PITTF&0x01)
    {
      del--;
      // Clear TOF flag
      PITTF = 0x01;
    }
  }
    
  // Stop Timer on channel 0  
  PITCFLMT = 0xA0;
}


/////////////////////////////////////////////////////////////////////////////////////////
// MAIN
// --------------------------------------------------------------------------------------
// 
/////////////////////////////////////////////////////////////////////////////////////////
void main(void) 
{

  TPeriphInit();
  
  for(;;)
  {
    #if(_TARGET)  // Send Buffer
      buf_send.id = 1;
      buf_send.RTR = FALSE;
      // Select AN00 as single input channel and single conversion
      ATD0CTL5 = 0x00;   
      while(!(ATD0STAT0 & 0x80))
        ; 
      buf_send.data[0] = ATD0DR0H;
      PORTA = buf_send.data[0];
      buf_send.len = 1; 
      (void)MSCANSendMsg(CAN_4, buf_send);
      
    #else       // Get Buffer
      if (MSCANCheckRcvdMsg(CAN_4))
        if(MSCANGetMsg(CAN_4, &buf_get)) {
          
          PORTB = buf_get.data[0];
          
          CAN4CTL0_SLPRQ = 1;         //sleep mode request
          while(CAN4CTL1_SLPAK == 0); //wait for acknowledge of sleep mode
          
          asm ANDCC #0x7F;  //clear S bit
          asm nop;
          asm STOP;         //STOP mode

          
        }
    #endif    
  }
  
}    
