#include <mc9s12xep100.h>
#include "mscan.h"

/////////////////////////////////////////////////////////////////////////////////////////
//
// fNBT = 125000 bit/s
// CANCLK = 4000000 Hz
// tBUS = 25 ns (0.5 m * 5*10^-9 sm^-1)
// tTx = 500 ns
// tRx = 500 ns
//
// NBT = 10
// Tq = 0.75ns
// P = 3
// SYNC_SEG = 1Tq
// PROP_SEG = 3Tq
// PHASE_SEG1 = 3Tq
// PHASE_SEG2 = 3Tq
// RJW = 3
// OscToll = 1.181102%

/////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
struct can_peripheral 
{
  Bool ena;
  unsigned char *init_reg;
} can_per[5] = 
  {
    FALSE, &CAN0CTL0,
    FALSE, &CAN1CTL0,
    FALSE, &CAN2CTL0,
    FALSE, &CAN3CTL0,
    FALSE, &CAN4CTL0,
  };

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
void MSCANOpenCommunication(unsigned char can_num)
{
  unsigned char *can_pt;

  can_per[can_num].ena = TRUE;
  can_pt = can_per[can_num].init_reg;
  if(!(can_pt[CANCTL1]&CAN0CTL1_INITAK_MASK)) 
  {
	  can_pt[CANCTL0] = CAN0CTL0_INITRQ_MASK;
	  while(!(can_pt[CANCTL1]&CAN0CTL1_INITAK_MASK))
		  ;
	}											   	
  can_pt[CANCTL1] = 0x80;	
  can_pt[CANBTR0] = 0x8B;	
  can_pt[CANBTR1] = 0x25;	
  can_pt[CANIDMR_1B+0] = 0xFF;
  can_pt[CANIDMR_1B+1] = 0xFF;
  can_pt[CANIDMR_1B+2] = 0xFF;
  can_pt[CANIDMR_1B+3] = 0xFF;
  can_pt[CANIDMR_2B+0] = 0xFF;
  can_pt[CANIDMR_2B+1] = 0xFF;
  can_pt[CANIDMR_2B+2] = 0xFF;
  can_pt[CANIDMR_2B+3] = 0xFF;
  can_pt[CANCTL0] = 0x00;
  
  //============
  CAN4CTL0_WUPE = 1;  //wake up enable
  CAN4RIER_WUPIE = 1; //wake up interrupt enable
  //============
  
  while(can_pt[CANCTL1]&CAN0CTL1_INITAK_MASK)
	  ;
  while(!(can_pt[CANCTL0]&CAN0CTL0_SYNCH_MASK))
	  ;
}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
void MSCANCloseCommunication(unsigned char can_num)
{
  unsigned char *can_pt;

  can_per[can_num].ena = FALSE;
  can_pt = can_per[can_num].init_reg;
  if(!(can_pt[CANCTL1]&CAN0CTL1_INITAK_MASK)) 
  {
	  can_pt[CANCTL0] = CAN0CTL0_INITRQ_MASK;
	  while(!(can_pt[CANCTL1]&CAN0CTL1_INITAK_MASK))
		  ;
	}
  can_pt[CANCTL1] = 0x00;	// CAN disables
}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
Bool MSCANSendMsg(unsigned char can_num, struct can_msg msg)
{
  
  unsigned char n_tx_buf = 0, i;
  unsigned char *can_pt;

  if(!can_per[can_num].ena)
    return(FALSE);
  can_pt = can_per[can_num].init_reg;
  if(msg.len > 8)
	  return(FALSE);
  if(!(can_pt[CANCTL0]&CAN0CTL0_SYNCH_MASK))
	  return(FALSE);
  while(!(can_pt[CANTFLG]&MaskOR(n_tx_buf)))
	  n_tx_buf = (n_tx_buf == MAX_TX_BUFFERS)? 0: (unsigned char)(n_tx_buf + 1);

  can_pt[CANTBSEL] = MaskOR(n_tx_buf);
    
  can_pt[CANTXIDR+0] = (unsigned char)(msg.id>>3);		// Only Standard Identifier is used
  can_pt[CANTXIDR+1] = (unsigned char)(msg.id<<5);		// Only Standard Identifier is used
  if(msg.RTR)
	  can_pt[CANTXIDR+1] |= 0x10;
  for(i = 0; i < msg.len; i++)
	  can_pt[CANTXDSR+i] = msg.data[i];
  can_pt[CANTXDLR] = msg.len;
  can_pt[CANTXTBPR] = msg.prty;
  can_pt[CANTFLG] = MaskOR(n_tx_buf);
    
  return(TRUE);

}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
Bool MSCANGetMsg(unsigned char can_num, struct can_msg *msg)
{

  unsigned char i;
  unsigned char *can_pt;

  if(!can_per[can_num].ena)
    return(FALSE);
  can_pt = can_per[can_num].init_reg;
  if(!(can_pt[CANRFLG]&CAN0RFLG_RXF_MASK))
	  return(FALSE);
  if(can_pt[CANRXIDR+1]&0x08)
	  return(FALSE);
  msg->id = ((can_pt[CANRXIDR+0]<<3)&0x0700) | (unsigned char)(can_pt[CANRXIDR+0]<<3) | (unsigned char)(can_pt[CANRXIDR+1]>>5);
  if(can_pt[CANRXIDR+1]&0x10)
	  msg->RTR = TRUE;
  else
	  msg->RTR = FALSE;
  msg->len = can_pt[CANRXDLR];
  for(i = 0; i < msg->len; i++)
	  msg->data[i] = can_pt[CANRXDSR+i];
  can_pt[CANRFLG] = CAN0RFLG_RXF_MASK;
  return(TRUE);

}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
Bool MSCANCheckRcvdMsg(unsigned char can_num)
{

  unsigned char *can_pt;

  if(!can_per[can_num].ena)
    return(FALSE);
  can_pt = can_per[can_num].init_reg;
  if(can_pt[CANRFLG]&CAN0RFLG_RXF_MASK)
	  return(TRUE);
  return(FALSE);

}

